// pecco -- please enjoy classification with conjunctive features
//  $Id: typedef.cpp 566 2011-08-29 12:37:46Z ynaga $
// Copyright (c) 2008-2011 Naoki Yoshinaga <ynaga@tkl.iis.u-tokyo.ac.jp>
#include "typedef.h"

namespace ny {
  // secure conversion from string to numerical
  template <> fl_t  strton <fl_t>  (const char* s, char** error)
#if defined (USE_FLOAT) || defined (ABUSE_TRIE)
  { return std::strtof (s, error); }
#else
  { return std::strtod (s, error); }
#endif
  template <> uint  strton <uint>  (const char* s, char** error) {
    uint64_t ret = std::strtoul (s, error, 10);
    if (ret > UINT_MAX)
      print_err (HERE "overflow: %s\n", s);
    return static_cast <uint> (ret);
  }
  template <> int   strton <int>   (const char* s, char** error) {
    int64_t ret = std::strtol (s, error, 10);
    if (ret > INT_MAX)      print_err (HERE "overflow: %s\n", s);
    else if (ret < INT_MIN) print_err (HERE "underflow: %s\n", s);
    return static_cast <int> (ret);
  }
  template <typename T> T strton (const char* s) {
    char* err;
    T n = strton <T> (s, &err);
    if (*err != '\0') print_err (HERE "invalid conversion: %s\n", s);
    return n;
  }
  // explict specialization
  template fl_t strton (const char* s);
  template uint strton (const char* s);
  template int  strton (const char* s);
}
